from dataclasses import dataclass
from typing import Union, List, Dict


@dataclass
class Vendor:
    name: str
    aliases: List[str] = None
    products: Union[str, Dict[str, dict]] = None
    open_source: Dict[str, str] = None
    services: str = None
    advisories: str = None

    def is_vendor(self) -> bool:
        return self.products is not None

    def is_open_source(self, is_github: bool = False) -> bool:
        if self.open_source is None:
            return False
        if is_github:
            return 'github' in self.open_source
        return True


@dataclass
class CNA:
    id: str
    name: str
    root: str
    email: str
    scope: Dict[str, Vendor]

    def get_owners(self):
        owners = []

        for vendor in self.scope.values():
            if vendor.is_open_source(is_github=True):
                owners.append(vendor.open_source['github'])

        return owners

    def is_vendor(self):
        return any(vendor.is_vendor() for vendor in self.scope.values())

    def is_open_source(self, is_github: bool = False):
        return any(vendor.is_open_source(is_github=is_github) for vendor in self.scope.values())
